@@ -61,6 +61,15 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
   else
     make_cmd += " -s --no-print-directory";
 
+  // NB: there appears to be no parallelism opportunity in the
+  // module-building makefiles, so while the following works, it
+  // doesn't seem to accomplish anything measurable as of F13.
+#if 0
+  long smp = sysconf(_SC_NPROCESSORS_ONLN);
+  if (smp > 1)
+    make_cmd += " -j " + lex_cast(smp);
+#endif
+
   if (strverscmp (s.kernel_base_release.c_str(), "2.6.29") < 0)
     {
       // Older kernels, before linux commit #fd54f502841c1, include
@@ -148,6 +157,8 @@ compile_pass (systemtap_session& s)
   //      mflags-y += -Iinclude/asm-x86/mach-default
   // but that path does not exist in an O= build tree.
   o << module_cflags << " += -Iinclude2/asm/mach-default" << endl;
+  if (s.kernel_source_tree != "")
+    o << module_cflags << " += -I" + s.kernel_source_tree << endl;
 
   // NB: don't try
   // o << module_cflags << " += -Iusr/include" << endl;
@@ -161,7 +172,6 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-hrtimer-getset-expires.c", "STAPCONF_HRTIMER_GETSET_EXPIRES", NULL);
   output_autoconf(s, o, "autoconf-inode-private.c", "STAPCONF_INODE_PRIVATE", NULL);
   output_autoconf(s, o, "autoconf-constant-tsc.c", "STAPCONF_CONSTANT_TSC", NULL);
-  output_autoconf(s, o, "autoconf-tsc-khz.c", "STAPCONF_TSC_KHZ", NULL);
   output_autoconf(s, o, "autoconf-ktime-get-real.c", "STAPCONF_KTIME_GET_REAL", NULL);
   output_autoconf(s, o, "autoconf-x86-uniregs.c", "STAPCONF_X86_UNIREGS", NULL);
   output_autoconf(s, o, "autoconf-nameidata.c", "STAPCONF_NAMEIDATA_CLEANUP", NULL);
@@ -181,7 +191,10 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-regset.c", "STAPCONF_REGSET", NULL);
   output_autoconf(s, o, "autoconf-utrace-regset.c", "STAPCONF_UTRACE_REGSET", NULL);
   output_autoconf(s, o, "autoconf-uprobe-get-pc.c", "STAPCONF_UPROBE_GET_PC", NULL);
+  output_exportconf(s, o, "tsc_khz", "STAPCONF_TSC_KHZ");
   output_exportconf(s, o, "cpu_khz", "STAPCONF_CPU_KHZ");
+  output_exportconf(s, o, "__module_text_address", "STAPCONF_MODULE_TEXT_ADDRESS");
+  output_exportconf(s, o, "add_timer_on", "STAPCONF_ADD_TIMER_ON");
 
   output_autoconf(s, o, "autoconf-probe-kernel.c", "STAPCONF_PROBE_KERNEL", NULL);
   output_autoconf(s, o, "autoconf-save-stack-trace.c",
@@ -192,6 +205,8 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-kallsyms-on-each-symbol.c", "STAPCONF_KALLSYMS_ON_EACH_SYMBOL", NULL);
   output_autoconf(s, o, "autoconf-walk-stack.c", "STAPCONF_WALK_STACK", NULL);
   output_autoconf(s, o, "autoconf-mm-context-vdso.c", "STAPCONF_MM_CONTEXT_VDSO", NULL);
+  output_autoconf(s, o, "autoconf-blk-types.c", "STAPCONF_BLK_TYPES", NULL);
+  output_autoconf(s, o, "autoconf-perf-structpid.c", "STAPCONF_PERF_STRUCTPID", NULL);
 
   o << module_cflags << " += -include $(STAPCONF_HEADER)" << endl;
 
@@ -220,10 +235,9 @@ compile_pass (systemtap_session& s)
 
   // o << "CFLAGS += -fno-unit-at-a-time" << endl;
 
-  // 512 bytes should be enough for anybody
-  // XXX but it's not enough for unwind_frame -- PR10821
-  // XXX temporarily bumping to 600 bytes
-  o << "EXTRA_CFLAGS += $(call cc-option,-Wframe-larger-than=600)" << endl;
+  // 256 bytes should be enough for anybody
+  // XXX this doesn't validate varargs, per gcc bug #41633
+  o << "EXTRA_CFLAGS += $(call cc-option,-Wframe-larger-than=256)" << endl;
 
   // Assumes linux 2.6 kbuild
   o << "EXTRA_CFLAGS += -Wno-unused" << (s.omit_werror ? "" : " -Werror") << endl;
@@ -418,14 +432,15 @@ uprobes_pass (systemtap_session& s)
   return rc;
 }
 
-int
-run_pass (systemtap_session& s)
+string
+make_run_command (systemtap_session& s, const string& module)
 {
   // for now, just spawn staprun
   string staprun_cmd = string(getenv("SYSTEMTAP_STAPRUN") ?: BINDIR "/staprun")
     + " "
     + (s.verbose>1 ? "-v " : "")
     + (s.verbose>2 ? "-v " : "")
+    + (s.suppress_warnings ? "-w " : "")
     + (s.output_file.empty() ? "" : "-o " + s.output_file + " ");
 
   if (s.cmd != "")
@@ -438,7 +453,7 @@ run_pass (systemtap_session& s)
     staprun_cmd += "-b " + lex_cast(s.buffer_size) + " ";
 
   if (s.need_uprobes)
-    staprun_cmd += "-u ";
+    staprun_cmd += "-u" + s.uprobes_path + " ";
 
   if (s.load_only)
     staprun_cmd += (s.output_file.empty() ? "-L " : "-D ");
@@ -446,20 +461,23 @@ run_pass (systemtap_session& s)
   if (!s.size_option.empty())
     staprun_cmd += "-S " + s.size_option + " ";
 
-  staprun_cmd += s.tmpdir + "/" + s.module_name + ".ko";
+  if (module.empty())
+    staprun_cmd += s.tmpdir + "/" + s.module_name + ".ko";
+  else
+    staprun_cmd += module;
 
   // add module arguments
   for (unsigned i=0; i<s.globalopts.size(); i++)
     staprun_cmd += " " + s.globalopts[i];
 
-  return stap_system (s.verbose, staprun_cmd);
+  return staprun_cmd;
 }
 
 
 // Build a tiny kernel module to query tracepoints
 int
 make_tracequery(systemtap_session& s, string& name,
-                const vector<string>& headers)
+                const vector<string>& decls)
 {
   static unsigned tick = 0;
   string basename("tracequery_kmod_" + lex_cast(++tick));
@@ -480,7 +498,13 @@ make_tracequery(systemtap_session& s, string& name,
   ofstream omf(makefile.c_str());
   // force debuginfo generation, and relax implicit functions
   omf << "EXTRA_CFLAGS := -g -Wno-implicit-function-declaration" << (s.omit_werror ? "" : " -Werror") << endl;
+  if (s.kernel_source_tree != "")
+    omf << "EXTRA_CFLAGS += -I" + s.kernel_source_tree << endl;
   omf << "obj-m := " + basename + ".o" << endl;
+
+  // RHBZ 655231: later rhel6 kernels' module-signing kbuild logic breaks out-of-tree modules
+  omf << "CONFIG_MODULE_SIG := n" << endl;
+
   omf.close();
 
   // create our source file
@@ -504,10 +528,11 @@ make_tracequery(systemtap_session& s, string& name,
   osrc << "#define DEFINE_TRACE(name, proto, args) \\" << endl;
   osrc << "  DECLARE_TRACE(name, TPPROTO(proto), TPARGS(args))" << endl;
 
-  // add the specified headers
-  for (unsigned z=0; z<headers.size(); z++)
+  // add the specified decls/#includes
+  for (unsigned z=0; z<decls.size(); z++)
     osrc << "#undef TRACE_INCLUDE_FILE\n"
-         << "#include <" << headers[z] << ">\n";
+         << "#undef TRACE_INCLUDE_PATH\n"
+         << decls[z] << "\n";
 
   // finish up the module source
   osrc << "#endif /* CONFIG_TRACEPOINTS */" << endl;
@@ -553,6 +578,9 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   ofstream omf(makefile.c_str());
   omf << "EXTRA_CFLAGS := -g -fno-eliminate-unused-debug-types" << endl;
 
+  // RHBZ 655231: later rhel6 kernels' module-signing kbuild logic breaks out-of-tree modules
+  omf << "CONFIG_MODULE_SIG := n" << endl;
+
   // NB: We use -include instead of #include because that gives us more power.
   // Using #include searches relative to the source's path, which in this case
   // is /tmp/..., so that's not helpful.  Using -include will search relative
