@@ -22,8 +22,9 @@
 #include "rpm_finder.h"
 #include "task_finder.h"
 #include "csclient.h"
+#include "remote.h"
 
-#include "sys/sdt.h"
+#include "stap-probe.h"
 
 #include <cstdlib>
 
@@ -41,24 +42,16 @@ extern "C" {
 
 using namespace std;
 
-static void uniq_list(list<string>& l)
+static void
+uniq_list(list<string>& l)
 {
-	list<string> r;
-	set<string> s;
-
-	for (list<string>::iterator i = l.begin(); i != l.end(); ++i) {
-		s.insert(*i);
-	}
-
-	for (list<string>::iterator i = l.begin(); i != l.end(); ++i) {
-		if (s.find(*i) != s.end()) {
-			s.erase(*i);
-			r.push_back(*i);
-		}
-	}
-
-	l.clear();
-	l.assign(r.begin(), r.end());
+  set<string> s;
+  list<string>::iterator i = l.begin();
+  while (i != l.end())
+    if (s.insert(*i).second)
+      ++i;
+    else
+      i = l.erase(i);
 }
 
 static void
@@ -82,7 +75,7 @@ printscript(systemtap_session& s, ostream& o)
           p->collect_derivation_chain (chain);
           probe* second = (chain.size()>1) ? chain[chain.size()-2] : chain[0];
 
-          #if 0  // dump everything about the derivation chain
+          if (s.verbose > 5) {
           p->printsig(cerr); cerr << endl;
           cerr << "chain[" << chain.size() << "]:" << endl;
           for (unsigned j=0; j<chain.size(); j++)
@@ -107,7 +100,7 @@ printscript(systemtap_session& s, ostream& o)
                     }
                 }
             }
-          #endif
+          }
 
           stringstream tmps;
           const probe_alias *a = second->get_alias();
@@ -301,9 +294,10 @@ int parse_kernel_config (systemtap_session &s)
   struct stat st;
   int rc = stat(kernel_config_file.c_str(), &st);
   if (rc != 0)
-    {
-	clog << "Checking \"" << kernel_config_file << "\" failed: " << strerror(errno) << endl
-	     << "Ensure kernel development headers & makefiles are installed." << endl;
+    {  
+	clog << "Checking \"" << kernel_config_file << "\" failed: " << strerror(errno) << endl;
+	find_devel_rpms(s, s.kernel_build_tree.c_str());
+	missing_rpm_list_print(s,"-devel");
 	return rc;
     }
 
@@ -411,7 +405,7 @@ static int
 passes_0_4 (systemtap_session &s)
 {
   int rc = 0;
-  
+
   // Create a temporary directory to build within.
   // Be careful with this, as "s.tmpdir" is "rm -rf"'d at the end.
   create_temp_dir (s);
@@ -419,13 +413,17 @@ passes_0_4 (systemtap_session &s)
   // Perform passes 0 through 4 using a compile server?
   if (! s.specified_servers.empty ())
     {
+#if HAVE_NSS
       compile_server_client client (s);
       return client.passes_0_4 ();
+#else
+      cerr << "WARNING: Without NSS, using a compile-server is not supported by this version of systemtap" << endl;
+#endif
     }
 
   // PASS 0: setting up
   s.verbose = s.perpass_verbose[0];
-  STAP_PROBE1(stap, pass0__start, &s);
+  PROBE1(stap, pass0__start, &s);
 
 
   // For PR1477, we used to override $PATH and $LC_ALL and other stuff
@@ -461,7 +459,7 @@ passes_0_4 (systemtap_session &s)
   // and reasonably timely exit.
   setup_signals(&handle_interrupt);
 
-  STAP_PROBE1(stap, pass0__end, &s);
+  PROBE1(stap, pass0__end, &s);
 
   struct tms tms_before;
   times (& tms_before);
@@ -469,7 +467,7 @@ passes_0_4 (systemtap_session &s)
   gettimeofday (&tv_before, NULL);
 
   // PASS 1a: PARSING USER SCRIPT
-  STAP_PROBE1(stap, pass1a__start, &s);
+  PROBE1(stap, pass1a__start, &s);
 
   struct stat user_file_stat;
   int user_file_stat_rc = -1;
@@ -521,7 +519,9 @@ passes_0_4 (systemtap_session &s)
   version_suffixes.push_back ("");
 
   // PASS 1b: PARSING LIBRARY SCRIPTS
-  STAP_PROBE1(stap, pass1b__start, &s);
+  PROBE1(stap, pass1b__start, &s);
+
+  set<pair<dev_t, ino_t> > seen_library_files;
 
   for (unsigned i=0; i<s.include_path.size(); i++)
     {
@@ -535,36 +535,49 @@ passes_0_4 (systemtap_session &s)
             rc ++;
           // GLOB_NOMATCH is acceptable
 
-          if (s.verbose>1 && globbuf.gl_pathc > 0)
-            clog << "Searched \"" << dir << "\", "
-                 << "found " << globbuf.gl_pathc << endl;
+          unsigned prev_s_library_files = s.library_files.size();
 
           for (unsigned j=0; j<globbuf.gl_pathc; j++)
             {
               if (pending_interrupts)
                 break;
 
-              // XXX: privilege only for /usr/share/systemtap?
-              stapfile* f = parse (s, globbuf.gl_pathv[j], true);
-              if (f == 0)
-                s.print_warning("tapset '" + string(globbuf.gl_pathv[j])
-                                + "' has errors, and will be skipped.");
-              else
-                s.library_files.push_back (f);
-
               struct stat tapset_file_stat;
               int stat_rc = stat (globbuf.gl_pathv[j], & tapset_file_stat);
               if (stat_rc == 0 && user_file_stat_rc == 0 &&
                   user_file_stat.st_dev == tapset_file_stat.st_dev &&
                   user_file_stat.st_ino == tapset_file_stat.st_ino)
                 {
-                  clog << "usage error: tapset file '" << globbuf.gl_pathv[j]
+                  cerr << "usage error: tapset file '" << globbuf.gl_pathv[j]
                        << "' cannot be run directly as a session script." << endl;
                   rc ++;
                 }
 
+              // PR11949: duplicate-eliminate tapset files
+              if (stat_rc == 0)
+                {
+                  pair<dev_t,ino_t> here = make_pair(tapset_file_stat.st_dev,
+                                                     tapset_file_stat.st_ino);
+                  if (seen_library_files.find(here) != seen_library_files.end())
+                    continue;
+                  seen_library_files.insert (here);
+                }
+
+              // XXX: privilege only for /usr/share/systemtap?
+              stapfile* f = parse (s, globbuf.gl_pathv[j], true);
+              if (f == 0 && !s.suppress_warnings)
+                s.print_warning("tapset '" + string(globbuf.gl_pathv[j])
+                                + "' has errors, and will be skipped.");
+              else
+                s.library_files.push_back (f);
             }
 
+          unsigned next_s_library_files = s.library_files.size();
+          if (s.verbose>1 && globbuf.gl_pathc > 0)
+            clog << "Searched \"" << dir << "\","
+                 << " found " << globbuf.gl_pathc 
+                 << " processed " << (next_s_library_files-prev_s_library_files) << endl;
+
           globfree (& globbuf);
         }
     }
@@ -614,7 +627,7 @@ passes_0_4 (systemtap_session &s)
          << "Try again with another '--vp 1' option."
          << endl;
 
-  STAP_PROBE1(stap, pass1__end, &s);
+  PROBE1(stap, pass1__end, &s);
 
   if (rc || s.last_pass == 1 || pending_interrupts) return rc;
 
@@ -623,7 +636,7 @@ passes_0_4 (systemtap_session &s)
 
   // PASS 2: ELABORATION
   s.verbose = s.perpass_verbose[1];
-  STAP_PROBE1(stap, pass2__start, &s);
+  PROBE1(stap, pass2__start, &s);
   rc = semantic_pass (s);
 
   if (s.listing_mode || (rc == 0 && s.last_pass == 2))
@@ -647,12 +660,15 @@ passes_0_4 (systemtap_session &s)
          << endl;
 
   /* Print out list of missing files.  XXX should be "if (rc)" ? */
-  missing_rpm_list_print(s);
+  missing_rpm_list_print(s,"-debuginfo");
 
-  STAP_PROBE1(stap, pass2__end, &s);
+  PROBE1(stap, pass2__end, &s);
 
   if (rc || s.listing_mode || s.last_pass == 2 || pending_interrupts) return rc;
 
+  rc = prepare_translate_pass (s);
+  if (rc || pending_interrupts) return rc;
+
   // Generate hash.  There isn't any point in generating the hash
   // if last_pass is 2, since we'll quit before using it.
   if (s.use_script_cache)
@@ -688,7 +704,7 @@ passes_0_4 (systemtap_session &s)
   s.verbose = s.perpass_verbose[2];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
-  STAP_PROBE1(stap, pass3__start, &s);
+  PROBE1(stap, pass3__start, &s);
 
   rc = translate_pass (s);
 
@@ -713,7 +729,7 @@ passes_0_4 (systemtap_session &s)
          << "Try again with another '--vp 001' option."
          << endl;
 
-  STAP_PROBE1(stap, pass3__end, &s);
+  PROBE1(stap, pass3__end, &s);
 
   if (rc || s.last_pass == 3 || pending_interrupts) return rc;
 
@@ -721,7 +737,7 @@ passes_0_4 (systemtap_session &s)
   s.verbose = s.perpass_verbose[3];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
-  STAP_PROBE1(stap, pass4__start, &s);
+  PROBE1(stap, pass4__start, &s);
 
   if (s.use_cache)
     {
@@ -771,13 +787,13 @@ passes_0_4 (systemtap_session &s)
 	}
     }
 
-  STAP_PROBE1(stap, pass4__end, &s);
+  PROBE1(stap, pass4__end, &s);
 
   return rc;
 }
 
 static int
-pass_5 (systemtap_session &s)
+pass_5 (systemtap_session &s, vector<remote*> targets)
 {
   // PASS 5: RUN
   s.verbose = s.perpass_verbose[4];
@@ -788,9 +804,13 @@ pass_5 (systemtap_session &s)
   // NB: this message is a judgement call.  The other passes don't emit
   // a "hello, I'm starting" message, but then the others aren't interactive
   // and don't take an indefinite amount of time.
-  STAP_PROBE1(stap, pass5__start, &s);
+  PROBE1(stap, pass5__start, &s);
   if (s.verbose) clog << "Pass 5: starting run." << endl;
-  int rc = run_pass (s);
+  int rc = 0; // XXX with multiple targets, need to deal with partial failure
+  for (unsigned i = 0; i < targets.size() && !pending_interrupts; ++i)
+    rc |= targets[i]->start();
+  for (unsigned i = 0; i < targets.size(); ++i)
+    rc |= targets[i]->finish();
   struct tms tms_after;
   times (& tms_after);
   unsigned _sc_clk_tck = sysconf (_SC_CLK_TCK);
@@ -808,7 +828,7 @@ pass_5 (systemtap_session &s)
     // Interrupting pass-5 to quit is normal, so we want an EXIT_SUCCESS below.
     pending_interrupts = 0;
 
-  STAP_PROBE1(stap, pass5__end, &s);
+  PROBE1(stap, pass5__end, &s);
 
   return rc;
 }
@@ -817,7 +837,11 @@ static void
 cleanup (systemtap_session &s, int rc)
 {
   // PASS 6: cleaning up
-  STAP_PROBE1(stap, pass6__start, &s);
+  PROBE1(stap, pass6__start, &s);
+
+  for (systemtap_session::session_map_t::iterator it = s.subsessions.begin();
+       it != s.subsessions.end(); ++it)
+    cleanup (*it->second, rc);
 
   // update the database information
   if (!rc && s.tapset_compile_coverage && !pending_interrupts) {
@@ -831,7 +855,7 @@ cleanup (systemtap_session &s, int rc)
   // Clean up temporary directory.  Obviously, be careful with this.
   remove_temp_dir (s);
 
-  STAP_PROBE1(stap, pass6__end, &s);
+  PROBE1(stap, pass6__end, &s);
 }
 
 int
@@ -851,16 +875,54 @@ main (int argc, char * const argv [])
   // If requested, query server status. This is independent of other tasks.
   query_server_status (s);
 
+  // If requested, manage trust of servers. This is independent of other tasks.
+  manage_server_trust (s);
+
   // Run the passes only if a script has been specified. The requirement for
   // a script has already been checked in systemtap_session::check_options.
   if (s.have_script)
     {
-      // Run passes 0-4, either locally or using a compile server.
-      rc = passes_0_4 (s);
+      vector<remote*> targets;
+      if (s.remote_uris.empty())
+	{
+	  remote* target = remote::create(s, "direct");
+	  if (target)
+	    targets.push_back(target);
+	  else
+	    rc = 1;
+	}
+      else
+	for (unsigned i = 0; i < s.remote_uris.size(); ++i)
+	  {
+	    remote *target = remote::create(s, s.remote_uris[i]);
+	    if (target)
+	      targets.push_back(target);
+	    else
+	      {
+		rc = 1;
+		break;
+	      }
+	  }
+
+      // Run passes 0-4 for each unique session,
+      // either locally or using a compile-server.
+      if (rc == 0)
+	{
+	  set<systemtap_session*> sessions;
+	  for (unsigned i = 0; i < targets.size(); ++i)
+	    sessions.insert(targets[i]->get_session());
+	  for (set<systemtap_session*>::iterator it = sessions.begin();
+	       it != sessions.end(); ++it)
+	    if ((rc = passes_0_4 (**it)))
+	      break;
+	}
 
       // Run pass 5, if requested
       if (rc == 0 && s.last_pass >= 5 && ! pending_interrupts)
-	rc = pass_5 (s);
+	rc = pass_5 (s, targets);
+
+      for (unsigned i = 0; i < targets.size(); ++i)
+	delete targets[i];
     }
 
   // Pass 6. Cleanup
