@@ -28,8 +28,11 @@ struct addr_map
   struct addr_map_entry entries[0];
 };
 
+#ifdef CONFIG_PREEMPT_RT
+static DEFINE_RAW_SPINLOCK(addr_map_lock);
+#else
 static DEFINE_SPINLOCK(addr_map_lock);
-
+#endif
 static struct addr_map* blackmap;
 
 /* Find address of entry where we can insert a new one. */
