@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Copy from user space functions
- * Copyright (C) 2005, 2006, 2007 Red Hat Inc.
+ * Copyright (C) 2005-2008 Red Hat Inc.
  * Copyright (C) 2005 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -9,11 +9,10 @@
  * later version.
  */
 
-#ifndef _COPY_C_ /* -*- linux-c -*- */
+#ifndef _COPY_C_		/* -*- linux-c -*- */
 #define _COPY_C_
 
 #include "string.c"
-
 /** @file copy.c
  * @brief Functions to copy from user space.
  */
@@ -26,8 +25,29 @@
  * @{
  */
 
-long _stp_strncpy_from_user(char *dst, const char __user *src, long count);
-//static long __stp_strncpy_from_user(char *dst, const char __user *src, long count);
+/** Safely read from userspace or kernelspace.
+ * On success, returns 0. Returns -EFAULT on error.
+ *
+ * This uses __get_user() to read from userspace or
+ * kernelspace.  Will not sleep or cause pagefaults when
+ * called from within a kprobe context.
+ *
+ * @param segment . KERNEL_DS for kernel access
+ *                  USER_DS for userspace.
+ */
+
+#define _stp_read_address(x, ptr, segment)    \
+	({				      \
+		long ret;		      \
+		mm_segment_t ofs = get_fs();  \
+		set_fs(segment);	      \
+		ret = __stp_get_user(x, ptr); \
+		set_fs(ofs);		      \
+		ret;   			      \
+	})
+
+
+static long __stp_strncpy_from_user(char *dst, const char __user *src, long count);
 
 #if defined (__i386__)
 #define __stp_strncpy_from_user(dst,src,count,res)			   \
@@ -85,7 +105,7 @@ do {									   \
 		: "i"(-EFAULT), "0"(count), "1"(count), "3"(src), "4"(dst) \
 		: "memory");						   \
 } while (0)
-#elif defined (__powerpc64__) || defined (__ia64__) || defined (__arm__)
+#elif defined (__powerpc__) || defined (__ia64__) || defined (__arm__)
 #define __stp_strncpy_from_user(dst,src,count,res) \
 	do { res = __strncpy_from_user(dst, src, count); } while(0)
 
@@ -110,8 +130,7 @@ do {									   \
  * <i>count</i> bytes and returns <i>count</i>.
  */
 
-long
-_stp_strncpy_from_user(char *dst, const char __user *src, long count)
+static long _stp_strncpy_from_user(char *dst, const char __user *src, long count)
 {
 	long res = -EFAULT;
 	if (access_ok(VERIFY_READ, src, count))
@@ -119,7 +138,6 @@ _stp_strncpy_from_user(char *dst, const char __user *src, long count)
 	return res;
 }
 
-
 /** Copy a block of data from user space.
  *
  * If some data could not be copied, this function will pad the copied
@@ -133,8 +151,7 @@ _stp_strncpy_from_user(char *dst, const char __user *src, long count)
  *
  */
 
-unsigned long
-_stp_copy_from_user (char *dst, const char __user *src, unsigned long count)
+static unsigned long _stp_copy_from_user(char *dst, const char __user *src, unsigned long count)
 {
 	if (count) {
 		if (access_ok(VERIFY_READ, src, count))
