@@ -234,7 +234,8 @@ int init_relayfs(void)
 	struct statfs st;
 	char rqbuf[128];
 	char buf[PATH_MAX], relay_filebase[PATH_MAX];
-
+        struct sigaction sa;
+        
 	dbug(2, "initializing relayfs\n");
 
 	reader[0] = (pthread_t)0;
@@ -277,6 +278,10 @@ int init_relayfs(void)
 		return -1;
 	}
 
+        /* PR7097 */
+        if (load_only)
+                return 0;
+
 	if (fsize_max) {
 		/* switch file mode */
 		for (i = 0; i < ncpus; i++) {
@@ -290,6 +295,9 @@ int init_relayfs(void)
 			if (outfile_name) {
 				/* special case: for testing we sometimes want to write to /dev/null */
 				if (strcmp(outfile_name, "/dev/null") == 0) {
+					/* This strcpy() is OK, since
+					 * we know buf is PATH_MAX
+					 * bytes long. */
 					strcpy(buf, "/dev/null");
 				} else {
 					len = stap_strfloctime(buf, PATH_MAX,
@@ -335,22 +343,19 @@ int init_relayfs(void)
 			out_fd[0] = STDOUT_FILENO;
 		
 	}
-	if (!load_only) {
-		struct sigaction sa;
 
-		sa.sa_handler = switchfile_handler;
-		sa.sa_flags = 0;
-		sigemptyset(&sa.sa_mask);
-		sigaction(SIGUSR2, &sa, NULL);
-		dbug(2, "starting threads\n");
-		for (i = 0; i < ncpus; i++) {
-			if (pthread_create(&reader[i], NULL, reader_thread,
-					   (void *)(long)i) < 0) {
-				_perr("failed to create thread");
-				return -1;
-			}
-		}
-	}
+        sa.sa_handler = switchfile_handler;
+        sa.sa_flags = 0;
+        sigemptyset(&sa.sa_mask);
+        sigaction(SIGUSR2, &sa, NULL);
+        dbug(2, "starting threads\n");
+        for (i = 0; i < ncpus; i++) {
+                if (pthread_create(&reader[i], NULL, reader_thread,
+                                   (void *)(long)i) < 0) {
+                        _perr("failed to create thread");
+                        return -1;
+                }
+        }
 	
 	return 0;
 }
