@@ -2,6 +2,8 @@
  *
  * staprun.c - SystemTap module loader
  *
+ * Copyright (C) 2005-2010 Red Hat, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -16,15 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Copyright (C) 2005-2009 Red Hat, Inc.
- *
  */
 
+#define _XOPEN_SOURCE
+#define _BSD_SOURCE
 #include "staprun.h"
+#include <string.h>
 #include <sys/uio.h>
 #include <glob.h>
-
-
+#include <time.h>
+#include <sys/prctl.h>
 
 /* used in dbug, _err and _perr */
 char *__name__ = "staprun";
@@ -32,6 +35,7 @@ char *__name__ = "staprun";
 extern long delete_module(const char *, unsigned int);
 
 int send_relocations ();
+int send_tzinfo ();
 
 
 static int run_as(int exec_p, uid_t uid, gid_t gid, const char *path, char *const argv[])
@@ -115,24 +119,16 @@ static int enable_uprobes(void)
 	if (run_as(0, uid, gid, argv[0], argv) == 0)
 		return 0;
 
-	/*
-	 * TODO: If user can't setresuid to root here, staprun will exit.
-	 * Is there a situation where that would fail but the subsequent
-	 * attempt to insert_module() would succeed?
-	 */
-	dbug(2, "Inserting uprobes module from /lib/modules, if any.\n");
-	i = 0;
-	argv[i++] = "/sbin/modprobe";
-	argv[i++] = "-q";
-	argv[i++] = "uprobes";
-	argv[i] = NULL;
-	if (run_as(0, 0, 0, argv[0], argv) == 0)
-		return 0;
+        /* NB: don't use /sbin/modprobe, without more env. sanitation. */
 
+	/* Try the specified module or the one from the runtime.  */
+	if (uprobes_path)
+	  snprintf (runtimeko, sizeof(runtimeko), "%s", uprobes_path);
+	else
+	  snprintf (runtimeko, sizeof(runtimeko), "%s/uprobes/uprobes.ko",
+		    (getenv("SYSTEMTAP_RUNTIME") ?: PKGDATADIR "/runtime"));
+	dbug(2, "Inserting uprobes module from %s.\n", runtimeko);
 	/* This module may be signed, so use insert_module to load it.  */
-	snprintf (runtimeko, sizeof(runtimeko), "%s/uprobes/uprobes.ko",
-		  (getenv("SYSTEMTAP_RUNTIME") ?: PKGDATADIR "/runtime"));
-	dbug(2, "Inserting uprobes module from SystemTap runtime %s.\n", runtimeko);
 	argv[0] = NULL;
 	if (insert_module(runtimeko, NULL, argv, assert_uprobes_module_permissions) == 0)
 		return 0;
@@ -179,6 +175,11 @@ static int remove_module(const char *name, int verb)
 	int ret;
 	dbug(2, "%s\n", name);
 
+#ifdef PR_SET_NAME
+        /* Make self easier to identify in vmcrash images */
+        prctl (PR_SET_NAME, "staprun-d");
+#endif
+
         (void) verb; /* XXX: ignore */
 
 	if (strcmp(name, "*") == 0) {
@@ -186,12 +187,19 @@ static int remove_module(const char *name, int verb)
 		return 0;
 	}
 
-        /* We could call init_ctl_channel / close_ctl_channel here, as a heuristic
-           to determine whether the module is being used by some other stapio process.
-           However, delete_module() does basically the same thing. */
+        /* We call init_ctl_channel/close_ctl_channel to check whether
+           the module is a systemtap-built one (having the right files),
+           and that it's already unattached (because otherwise it'd EBUSY
+           the opens. */
+        ret = init_ctl_channel (name, 0);
+        if (ret < 0) {
+                err("Error, '%s' is not a zombie systemtap module.\n", name);
+                return ret;
+        }
+        close_ctl_channel ();
 
 	dbug(2, "removing module %s\n", name);
-	STAP_PROBE1(staprun, remove__module, name);
+	PROBE1(staprun, remove__module, name);
 	ret = delete_module (name, O_NONBLOCK);
 	if (ret != 0) {
                 /* XXX: maybe we should just accept this, with a
@@ -223,7 +231,7 @@ int init_staprun(void)
                      without first removing the kernel module.  This would block
                      a subsequent rerun attempt.  So here we gingerly try to
                      unload it first. */
-		  int ret = delete_module (modname, O_NONBLOCK);
+                  int ret = remove_module (modname, 0);
 		  err("Retrying, after attempted removal of module %s (rc %d)\n", modname, ret);
 		  /* Then we try an insert a second time.  */
 		  if (insert_stap_module() < 0)
@@ -231,6 +239,8 @@ int init_staprun(void)
 		}
 		if (send_relocations() < 0)
 			return -1;
+                if (send_tzinfo() < 0)
+                        return -1;
 	}
 	return 0;
 }
@@ -328,13 +338,17 @@ void send_a_relocation (const char* module, const char* reloc, unsigned long lon
 {
   struct _stp_msg_relocation msg;
 
-  if (strlen(module) >= STP_MODULE_NAME_LEN)
-    { _perr ("module name too long: %s", module); return; }
-  strcpy (msg.module, module);
-
-  if (strlen(reloc) >= STP_SYMBOL_NAME_LEN)
-    { _perr ("reloc name too long: %s", reloc); return; }
-  strcpy (msg.reloc, reloc);
+  if (strlen(module) >= STP_MODULE_NAME_LEN-1) {
+          dbug (1, "module name too long: %s", module);
+          return; 
+  }
+  strncpy (msg.module, module, STP_MODULE_NAME_LEN);
+  
+  if (strlen(reloc) >= STP_SYMBOL_NAME_LEN-1) {
+          dbug (1, "reloc name too long: %s", module);
+          return; 
+  }
+  strncpy (msg.reloc, reloc, STP_MODULE_NAME_LEN);
 
   msg.address = address;
 
@@ -343,10 +357,14 @@ void send_a_relocation (const char* module, const char* reloc, unsigned long lon
 }
 
 
+#ifdef __powerpc64__
+#define KERNEL_RELOC_SYMBOL ".__start"
+#else
+#define KERNEL_RELOC_SYMBOL "_stext"
+#endif
+
 int send_relocation_kernel ()
 {
-  int srkrc = 0;
-
   FILE* kallsyms = fopen ("/proc/kallsyms", "r");
   if (kallsyms == NULL)
     {
@@ -356,28 +374,19 @@ int send_relocation_kernel ()
   else
     {
       int done_with_kallsyms = 0;
+      char *line = NULL;
+      size_t linesz = 0;
       while (! feof(kallsyms) && !done_with_kallsyms)
         {
-          char *line = NULL;
-          size_t linesz = 0;
           ssize_t linesize = getline (& line, & linesz, kallsyms);
-          if (linesize < 0)
-            break;
-          else
+          if (linesize > 0)
             {
               unsigned long long address;
-              char type;
-              char* symbol = NULL;
-              int rc = sscanf (line, "%llx %c %as", &address, &type, &symbol);
-              free (line); line=NULL;
-              if (symbol == NULL) continue; /* OOM? */
-
-#ifdef __powerpc64__
-#define KERNEL_RELOC_SYMBOL ".__start"
-#else
-#define KERNEL_RELOC_SYMBOL "_stext"
-#endif
-              if ((rc == 3) && (0 == strcmp(symbol,KERNEL_RELOC_SYMBOL)))
+	      int pos = -1;
+	      if (sscanf (line, "%llx %*c %n", &address, &pos) == 1
+		  && pos != -1
+		  && linesize - pos == sizeof KERNEL_RELOC_SYMBOL
+		  && !strcmp(line + pos, KERNEL_RELOC_SYMBOL "\n"))
                 {
                   /* NB: even on ppc, we use the _stext relocation name. */
                   send_a_relocation ("kernel", "_stext", address);
@@ -385,20 +394,21 @@ int send_relocation_kernel ()
                   /* We need nothing more from the kernel. */
                   done_with_kallsyms=1;
                 }
-
-              free (symbol);
             }
         }
+      free (line);
       fclose (kallsyms);
-      if (!done_with_kallsyms) srkrc = -1;
-      /* detect note section, send flag if there 
+      if (!done_with_kallsyms)
+	return -1;
+
+      /* detect note section, send flag if there
        * NB: address=2 represents existed note, the real one in _stp_module
-       */ 
-      if (srkrc != -1 && !access("/sys/kernel/notes", R_OK))
+       */
+      if (!access("/sys/kernel/notes", R_OK))
 	 send_a_relocation ("kernel", ".note.gnu.build-id", 2);
     }
 
-  return srkrc;
+  return 0;
 }
 
 
@@ -426,24 +436,24 @@ void send_relocation_modules ()
          Sample gl_pathv[]: /sys/modules/zlib_deflate/sections/.text
          Pieces:                         ^^^^^^^^^^^^          ^^^^^
       */
-      section_name = rindex (module_section_file, '/');
+      section_name = strrchr (module_section_file, '/');
       if (! section_name) continue;
       section_name ++;
 
       if (!strcmp (section_name, ".")) continue;
       if (!strcmp (section_name, "..")) continue;
 
-      module_name = index (module_section_file, '/');
+      module_name = strchr (module_section_file, '/');
       if (! module_name) continue;
       module_name ++;
-      module_name = index (module_name, '/');
+      module_name = strchr (module_name, '/');
       if (! module_name) continue;
       module_name ++;
-      module_name = index (module_name, '/');
+      module_name = strchr (module_name, '/');
       if (! module_name) continue;
       module_name ++;
 
-      module_name_end = index (module_name, '/');
+      module_name_end = strchr (module_name, '/');
       if (! module_name_end) continue;
 
       secfile = fopen (module_section_file, "r");
@@ -489,3 +499,32 @@ int send_relocations ()
  out:
   return rc;
 }
+
+
+int send_tzinfo ()
+{
+  int rc;
+  struct _stp_msg_tzinfo tzi;
+  time_t now_t;
+  struct tm* now;
+
+  rc = init_ctl_channel (modname, 0);
+  if (rc < 0) goto out;
+
+  /* NB: This is not good enough; it sends DST-unaware numbers. */
+#if 0
+  tzset ();
+  tzi.tz_gmtoff = timezone;
+  strncpy (tzi.tz_name, tzname[0], STP_TZ_NAME_LEN);
+#endif
+
+  time (& now_t);
+  now = localtime (& now_t);
+  tzi.tz_gmtoff = - now->tm_gmtoff;
+  strncpy (tzi.tz_name, now->tm_zone, STP_TZ_NAME_LEN);
+
+  send_request(STP_TZINFO, & tzi, sizeof(tzi));
+  close_ctl_channel ();
+ out:
+  return rc;
+}
