@@ -179,7 +179,7 @@ itrace_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "struct stap_itrace_probe {";
   s.op->indent(1);
   s.op->newline() << "struct stap_task_finder_target tgt;";
-  s.op->newline() << "struct stap_probe probe;";
+  s.op->newline() << "struct stap_probe * const probe;";
   s.op->newline() << "int single_step;";
   s.op->newline(-1) << "};";
   s.op->newline() << "static void enter_itrace_probe(struct stap_itrace_probe *p, struct pt_regs *regs, void *data);";
@@ -195,7 +195,7 @@ itrace_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "c->data = data;";
 
   // call probe function
-  s.op->newline() << "(*p->probe.ph) (c);";
+  s.op->newline() << "(*p->probe->ph) (c);";
   common_probe_entryfn_epilogue (s.op);
 
   s.op->newline() << "return;";
