@@ -101,7 +101,7 @@ timer_derived_probe_group::emit_module_decls (systemtap_session& s)
 
   s.op->newline() << "static struct stap_timer_probe {";
   s.op->newline(1) << "struct timer_list timer_list;";
-  s.op->newline() << "struct stap_probe probe;";
+  s.op->newline() << "struct stap_probe * const probe;";
   s.op->newline() << "unsigned intrv, ms, rnd;";
   s.op->newline(-1) << "} stap_timer_probes [" << probes.size() << "] = {";
   s.op->indent(1);
@@ -127,7 +127,7 @@ timer_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline(-1) << "{";
   s.op->indent(1);
   common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "stp->probe");
-  s.op->newline() << "(*stp->probe.ph) (c);";
+  s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s.op);
   s.op->newline(-1) << "}";
   s.op->newline(-1) << "}";
@@ -141,7 +141,7 @@ timer_derived_probe_group::emit_module_init (systemtap_session& s)
 
   s.op->newline() << "for (i=0; i<" << probes.size() << "; i++) {";
   s.op->newline(1) << "struct stap_timer_probe* stp = & stap_timer_probes [i];";
-  s.op->newline() << "probe_point = stp->probe.pp;";
+  s.op->newline() << "probe_point = stp->probe->pp;";
   s.op->newline() << "init_timer (& stp->timer_list);";
   s.op->newline() << "stp->timer_list.function = & enter_timer_probe;";
   s.op->newline() << "stp->timer_list.data = i;"; // NB: important!
@@ -268,7 +268,7 @@ hrtimer_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "static unsigned long stap_hrtimer_resolution;"; // init later
   s.op->newline() << "static struct stap_hrtimer_probe {";
   s.op->newline(1) << "struct hrtimer hrtimer;";
-  s.op->newline() << "struct stap_probe probe;";
+  s.op->newline() << "struct stap_probe * const probe;";
   s.op->newline() << "int64_t intrv, rnd;";
   s.op->newline(-1) << "} stap_hrtimer_probes [" << probes.size() << "] = {";
   s.op->indent(1);
@@ -315,7 +315,7 @@ hrtimer_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "{";
   s.op->indent(1);
   common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "stp->probe");
-  s.op->newline() << "(*stp->probe.ph) (c);";
+  s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s.op);
   s.op->newline(-1) << "}";
   s.op->newline() << "return rc;";
@@ -336,7 +336,7 @@ hrtimer_derived_probe_group::emit_module_init (systemtap_session& s)
 
   s.op->newline() << "for (i=0; i<" << probes.size() << "; i++) {";
   s.op->newline(1) << "struct stap_hrtimer_probe* stp = & stap_hrtimer_probes [i];";
-  s.op->newline() << "probe_point = stp->probe.pp;";
+  s.op->newline() << "probe_point = stp->probe->pp;";
   s.op->newline() << "hrtimer_init (& stp->hrtimer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);";
   s.op->newline() << "stp->hrtimer.function = & enter_hrtimer_probe;";
   // There is no hrtimer field to identify *this* (i-th) probe handler
@@ -427,7 +427,7 @@ profile_derived_probe_group::emit_module_decls (systemtap_session& s)
   // open-code the same logic here.
 
   s.op->newline() << "static void enter_all_profile_probes (struct pt_regs *regs) {";
-  s.op->newline(1) << "static const struct stap_probe probe = "
+  s.op->newline(1) << "struct stap_probe * probe = "
                    << common_probe_init (probes[0]) << ";";
   common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "probe");
   s.op->newline() << "c->regs = regs;";
@@ -441,13 +441,14 @@ profile_derived_probe_group::emit_module_decls (systemtap_session& s)
         {
           // Some lightweight inter-probe context resetting
           // XXX: not quite right: MAXERRORS not respected
+          // XXX: STP_TIMING stats are also not correct
+          s.op->newline() << "probe = " << common_probe_init (probes[i]) << ";";
+          s.op->newline() << "#ifdef STP_NEED_PROBE_NAME";
+          s.op->newline() << "c->probe_name = probe->pn;";
+          s.op->newline() << "#endif";
           s.op->newline() << "c->actionremaining = MAXACTION;";
         }
-      s.op->newline() << "#ifdef STP_NEED_PROBE_NAME";
-      s.op->newline() << "c->probe_name = "
-                      << lex_cast_qstring (*probes[i]->script_location()) << ";";
-      s.op->newline() << "#endif";
-      s.op->newline() << "if (c->last_error == NULL) " << probes[i]->name << " (c);";
+      s.op->newline() << "if (c->last_error == NULL) probe->ph (c);";
     }
   common_probe_entryfn_epilogue (s.op);
   s.op->newline(-1) << "}";
